/**
 * Functions dealing with DARIAH-DE Storage upload, download, sharing, unsharing,
 * and deleting.
 *
 * TODO: jquery object
 */

/*
 * Create new file, get storage file ID.
 */
function postToDariahStorage(postdata) {
    // Check for migration from old storage.
    if (isOldStorageID(dsid)) {
        sessionStorage.setItem('migration', true);
    } else {
        sessionStorage.removeItem('migration');
    }
    // Assemble bearer token and logID.
    var token = bearerPrefix + sessionStorage.getItem('tok');
    var logID = logIDPrefix + (new Date()).getMilliseconds();
    showMessage('console', '...creating...');
    // Create new file.
    $.ajax({
		url: storageURL,
		type: 'POST',
        headers: { 'Authorization': token, 'X-Transaction-ID': logID },
		contentType: 'text/csv',
		data: postdata,
		success: function(data, status, xhr) {
			//console.log('loc: ' + xhr.getResponseHeader('Location'));
			var location = xhr.getResponseHeader('Location');
			dsid = location.substring(location.lastIndexOf('/') + 1);
            // Reload dataset if just migrated.
            if (sessionStorage.getItem('migration') === 'true') {
                document.location.href = host + '/edit/index.html?id=' + dsid;
            }
            // No migration: Just change the URL here, do NO reload of page to stay in correkt workflow!
            // Sorry for the urglimurgli! Really don't know what it's supposed to mean!
            else {
                history.replaceState(
                    { urgli: 'murgli' },
                    'id', window.location.origin + window.location.pathname + '?id=' + dsid
                );
                newSheetAlert();
                doThingsOnIDExistingCheck();
            }
		},
        error: function(xhr, textStatus, errorThrown) {
            var title = 'Error creating dataset: ' + xhr.status + ' ' + errorThrown + '!';
            var message = '<p>No new dataset could be created!</p>';
            newAlert('error', title,  message);
            // Re-set storage ID.
        	dsid = "";
		}
	});
}

/**
 * Load file from DARIAH-DE storage using storage ID.
 */
function loadFromDariahStorage(id) {
    // Assemble bearer token and logID.
    var token = bearerPrefix + sessionStorage.getItem('tok');
    var logID = logIDPrefix + (new Date()).getMilliseconds();
    showMessage('console', '...loading...');
    $.ajax({
		url: storageURL + id,
		type: 'GET',
        headers: { 'Authorization': token, 'X-Transaction-ID': logID },
		success: function(data) {
            doThingsOnIDExistingCheck();
			csvToTable(data);
		},
        error: function(xhr, textStatus, errorThrown) {
            // Have we got a token already? If not, just do authenticate first!
            if (sessionStorage.getItem('tok') === null) {
                authenticate('id=' + id);
            }
            // If a token does exist and no read access is granted, just give the correct error.
            else {
                var title = 'Error loading dataset: ' + xhr.status + ' ' + errorThrown + '!';
                var message = '<p>The dataset with ID <i>' + id + '</i> could not be loaded from the DARIAH-DE Storage!</p>';
                newAlert('error', title,  message);
                // Re-set storage ID.
        	    dsid = "";
                doThingsOnIDExistingCheck();
                showFirstStartWizard();
            }
		}
	});
}

/**
 * Load file from old DARIAH-DE storage, set button states accordingly.
 */
function loadFromOldDariahStorage(id) {
    showMessage('console', '...creating...');
    $.ajax({
 		url: oldStorageURL + id,
 		type: 'GET',
 		success: function(data) {
            doThingsOnOldStorageID();
 			csvToReadOnlyTable(data);
            var title = 'This dataset is now read-only!';
            var message = '<p>The dataset with ID <i>' + id + '</i> is still stored in the old DARIAH-DE Storage and has been rendered read-only! You can either just download it using the <strong>download dataset</strong> button on the right, <strong>view the dataset</strong> with the Geo-Browser, or just <strong>create a new dataset</strong> in the DARIAH-DE OwnStorage to migrate and then further edit your data and share it if you wish.</p>';
            newAlert('warning', title,  message);
 		},
         error: function(xhr, textStatus, errorThrown) {
            var title = 'Error loading dataset: ' + xhr.status + ' ' + errorThrown + '!';
            var message = '<p>The dataset with ID <i>' + id + '</i> could not be loaded from the DARIAH-DE Storage!</p>';
            newAlert('error', title,  message);
            // Re-set storage ID.
         	dsid = "";
            doThingsOnIDExistingCheck();
            showFirstStartWizard();
 		}
 	});
 }

/**
 * Load file from old DARIAH-DE storage using storage ID, store data to OwnStorage.
 */
function migrateOldDataset(id) {
    showMessage('console', '...migrating...');
    $.ajax({
 		url: oldStorageURL + id,
 		type: 'GET',
 		success: function(data) {
            postToDariahStorage(data);
 		},
        error: function(xhr, textStatus, errorThrown) {
            var title = 'Error loading dataset: ' + xhr.status + ' ' + errorThrown + '!';
            var message = '<p>The dataset with ID <i>' + id + '</i> could not be loaded from the DARIAH-DE Storage!</p>';
            newAlert('error', title,  message);
            // Re-set storage ID.
         	dsid = "";
            doThingsOnIDExistingCheck();
            showFirstStartWizard();
 		}
 	});
}

/**
 * Load file from DARIAH-DE storage using storage ID, check shared status.
 */
function checkSharedStatus(id) {
    // Assemble logID.
    var logID = logIDPrefix + (new Date()).getMilliseconds();
    $.ajax({
		url: storageURL + id,
		type: 'GET',
        headers: { 'X-Transaction-ID': logID },
		success: function(data) {
            // Public datasets already are shared!
            setSharedStatus();
		},
        error: function(xhr, textStatus, errorThrown) {
            // Nonpublic datasets are not yet shared, so we are allowed to share them.
            setUnsharedStatus();
        }
	});
}

/*
 * Update a dataset (file) using ID and post data.
 */
function updateDariahStorage(id, postdata) {
    // Assemble bearer token and logID.
    var token = bearerPrefix + sessionStorage.getItem('tok');
    var logID = logIDPrefix + (new Date()).getMilliseconds();

    var updateError1 = '<p>The Dataset with ID <i>';
    var updateError2 = '</i> could not be updated to the DARIAH-DE Storage!</p>';

    // Check for shared status first, if true, we have to unshare, write, and share again!
    // ARGL! :-)
    if (sessionStorage.getItem('public') === 'true') {
        var error = false;
        var status;
        var thrown;
        // UNSHARE if shared!
        showMessage('console', '...unsharing...');
        $.ajax({
        	url: storageURL + id + "/unpublish",
            type: 'POST',
            headers: { 'Authorization': token, 'X-Transaction-ID': logID },
        	success: function(response) {
                // UPDATE if unshared!
                showMessage('console', '...writing...');
                $.ajax({
            		url: storageURL + id,
            		type: 'PUT',
                    headers: { 'Authorization': token, 'X-Transaction-ID': logID },
            		contentType: 'text/csv',
            		data: postdata,
                    success: function(response) {
                        showMessage('console', '...sharing again...');
                        // SHARE if updated!
                        $.ajax({
                    		url: storageURL + id + "/publish",
                            type: 'POST',
                            headers: { 'Authorization': token, 'X-Transaction-ID': logID },
                    		success: function(response) {
                                showMessage('console', 'All changes saved.');
                    		},
                            error: function(xhr, textStatus, errorThrown) {
                                var title = 'Error updating dataset (share): ' + xhr.status + ' ' + errorThrown + '!';
                                var message = updateError1 + id + updateError2;
                                newAlert('error', title,  message);
                            }
                    	});
            		},
                    error: function(xhr, textStatus, errorThrown) {
                        var title = 'Error updating dataset: ' + xhr.status + ' ' + errorThrown + '!';
                        var message = updateError1 + id + updateError2;
                        newAlert('error', title,  message);
                    }
            	});
        	},
            error: function(xhr, textStatus, errorThrown) {
                var title = 'Error updating dataset (unshare): ' + xhr.status + ' ' + errorThrown + '!';
                var message = updateError1 + id + updateError2;
                newAlert('error', title,  message);
            }
        });
        if (error === true) {
            var title = 'Error updating dataset: ' + status + ' ' + thrown + '!';
            var message = updateError1 + id + updateError2;
            newAlert('error', title,  message);
        }
    }
    // Just do UPDATE if not shared!
    else {
        showMessage('console', '...saving...');
        $.ajax({
            url: storageURL + id,
            type: 'PUT',
            headers: { 'Authorization': token, 'X-Transaction-ID': logID },
            contentType: 'text/csv',
            data: postdata,
            success: function(response) {
                showMessage('console', 'All changes saved.');
            },
            error: function(xhr, textStatus, errorThrown) {
                var title = 'Error updating dataset: ' + xhr.status + ' ' + errorThrown + '!';
                var message = updateError1 + id + updateError2;
                newAlert('error', title,  message);
            }
        });
    }
}

/**
 * Share a dataset using the publish() method of the DARIAH-DE PDP using DARIAH-DE OwnStorage.
 */
function publishDataset() {
    // Assemble bearer token and logID.
    var token = bearerPrefix + sessionStorage.getItem('tok');
    var logID = logIDPrefix + (new Date()).getMilliseconds();
    // Share file.
    $.ajax({
		url: storageURL + dsid + "/publish",
        type: 'POST',
        headers: { 'Authorization': token, 'X-Transaction-ID': logID },
		success: function(response) {
            var title = 'Dataset successfully shared!';
            var accessURL =  storageURL + dsid;
            var message = '<p>The Dataset with ID <i>' + dsid + '</i> has been successfully shared.<br/> It can now be read by everyone using the URL <a href="' + accessURL + '">' + accessURL + '</a> by tools such as the DARIAH-DE Geo-Browser or simply for downloading via browser.</p>';
            newAlert('success', title,  message);
            setSharedStatus();
		},
        error: function(xhr, textStatus, errorThrown) {
            var title = 'Error sharing dataset: ' + xhr.status + ' ' + errorThrown + '!';
            var message = '<p>The Dataset with ID <i>' + dsid + '</i> could not be shared! Maybe it has to be unshared first!</p>';
            newAlert('error', title,  message);
		}
	});
}

/**
 * Unshare a dataset (again) using the unpublish() method of the DARIAH-DE PDP using DARIAH-DE OwnStorage.
 */
function unpublishDataset() {
    // Assemble bearer token and logID.
    var token = bearerPrefix + sessionStorage.getItem('tok');
    var logID = logIDPrefix + (new Date()).getMilliseconds();
    // Unshare file.
    $.ajax({
		url: storageURL + dsid + "/unpublish",
        type: 'POST',
        headers: { 'Authorization': token, 'X-Transaction-ID': logID },
		success: function(response) {
            setUnsharedStatus();
            var title = 'Dataset successfully unshared!';
            var message = '<p>The dataset with ID <i>' + dsid + '</i> has been successfully unshared.<br/> It can now only accessed by you again!</p>';
            newAlert('success', title,  message);
		},
        error: function(xhr, textStatus, errorThrown) {
            var title = 'Error unsharing dataset: ' + xhr.status + ' ' + errorThrown + '!';
            var message = '<p>The dataset with ID <i>' + dsid + '</i> could not be unshared!</p>';
            newAlert('error', title,  message);
		}
	});
}

/**
 * Unpublish a dataset first, then delete.
 */
function unpublishAndDeleteDataset() {
    // Assemble bearer token and logID.
    var token = bearerPrefix + sessionStorage.getItem('tok');
    var logID = logIDPrefix + (new Date()).getMilliseconds();
    // Delete file.
    $.ajax({
		url: storageURL + dsid + "/unpublish",
        type: 'POST',
        headers: { 'Authorization': token, 'X-Transaction-ID': logID },
		success: function(response) {
            deleteDatasetFromStorage();
		},
        error: function(xhr, textStatus, errorThrown) {
            deleteDatasetFromStorage();
		}
	});
}

/**
 * Delete a dataset from the DARIAH-DE OwnStorage.
 */
function deleteDatasetFromStorage() {
    // Assemble bearer token and logID.
    var token = bearerPrefix + sessionStorage.getItem('tok');
    var logID = logIDPrefix + (new Date()).getMilliseconds();
    // Delete file.
	$.ajax({
		url: storageURL + dsid,
		type: 'DELETE',
        headers: { 'Authorization': token, 'X-Transaction-ID': logID },
		success: function(response) {
            var title = 'Dataset successfully deleted!';
            var message = '<p>The dataset with ID <i>' + dsid + '</i> has been successfully deleted from the DARIAH-DE Storage.</p>';
            newAlert('success', title,  message);
            // Re-set storage ID.
        	dsid = "";
            // Remove token hash from URL.
            window.location = window.location.origin + window.location.pathname;
		},
        error: function(xhr, textStatus, errorThrown) {
            checkSharedStatus(dsid);
            var title = 'Error deleting dataset: ' + xhr.status + ' ' + errorThrown + '!';
            var message = '<p>The dataset with ID <i>' + dsid + '</i> could not be deleted from the DARIAH-DE Storage!</p>';
            newAlert('error', title,  message);
		}
	});
}

/**
 * Load user's file OwnStorage list from DARIAH-DE storage to manage user's OwnStorage files.
 *
 * TODO Not yet finished and still in beta state: See how we can recognize Geo-Browser objects
 * in OwnStorage and distinguish them from objects such as files from DARIAH-DE Repository or
 * Publikator?
 */
function loadOwnStorageFileList() {
    // Assemble bearer token and logID.
    var token = bearerPrefix + sessionStorage.getItem('tok');
    var logID = logIDPrefix + (new Date()).getMilliseconds();
    $.ajax({
		url: storageURL + "/list",
		type: 'GET',
        headers: { 'Authorization': token, 'X-Transaction-ID': logID },
		success: function(data) {
            console.log("Files: ", data);
		},
        error: function(xhr, textStatus, errorThrown) {
            // Have we got a token already? If not, just do authenticate first!
            if (sessionStorage.getItem('tok') === null) {
                // authenticate('id=' + id);
                console.log("NO AUTH");
            }
            // If a token does exist and no read access is granted, just give the correct error.
            else {
                var title = 'Error loading file list: ' + xhr.status + ' ' + errorThrown + '!';
                var message = '<p>The file list could not be loaded from the DARIAH-DE Storage!</p>';
                newAlert('error', title,  message);
            }
		}
	});
}

/**
 * Check if token is existing, if not, get one! action must always contain kay AND value,
 * such as 'action=create' or 'id=12345'!
 */
function authenticate(action) {
    // Show redirection message.
    var title = 'Redirecting to DARIAH AAI';
    var message = '<p>You are being redirected to the DARIAH AAI to get you a DARIAH-DE storage token for OwnStorage access.</p>';
    newAlert('success', title,  message);
    login(action);
}

/*
 * Sets the OAuth token from token hash, if existing.
 */
function setToken() {
    // Get complete token hash from PDP.
    var acctok = window.location.hash.substr(1);
    // Get pure token out of complete token hash.
    var puretok = acctok.substr(acctok.indexOf("access_token=") + 13, acctok.indexOf("&") - 13);
    sessionStorage.setItem('tok', puretok);
    // Remove token hash from URL.
    window.location = window.location.origin + window.location.pathname + window.location.search;
}

/*
 * Check and fill login/logout menu.
 */
function checkLoginLogoutMenu() {
    if (sessionStorage.getItem('tok')) {
        $('#loggedin').append('<i class="icon-signout icon-white"></i> Logout');
        $('#loggedin').attr('href', 'javascript:logout()');
    } else {
        $('#loggedin').append('<i class="icon-signin icon-white"></i> Login');
        $('#loggedin').attr('href', 'javascript:login()');
    }
}

/**
 * Redirect to PDP and login.
 */
function login(action) {
    var questionmark = '';
    // Only use '?' if action is undefined.
    if (action) {
        questionmark = '?' + action;
    }
    window.location = pdpURL + window.location.protocol + "//" + window.location.hostname + window.location.pathname + questionmark;
}

/**
 * Logout: Just delete the token from session store and go to index page.
 */
function logout() {
    sessionStorage.removeItem('tok');
    window.location = window.location.origin + window.location.pathname;
}
