/**
 * Global variables
 */
var dsid; // dariah storage id
var map; // osm
var proj4326; // osm projection
var projmerc; // mercator projection
var locationPickerPoint;
var locationPickerLayer;
var dragFeature;

/**
 * Stuff to setup when DOM is ready.
 */
$(document).ready(function() {
    checkLoginLogoutMenu();
    initTable();
    initMap(); // Init the OpenStreetmap widget.
    initClipboard(); // Init clipboard and tooltips.
    init();
    showNews();
});

/**
 * Main init method, called every time the page is reloaded!
 */
function init() {

    // Drag and drop handler for CSV documents to workarea.
    $('#workArea')
        .bind('dragover', handleDragOver, false)
        .bind('drop', handleDropFileSelect);

    // Action for fileuploadbutton.
    $('#files').bind('change', handleFileSelect);

    // jQuery creates it's own event object, and it doesn't have a dataTransfer property yet.
    // This adds dataTransfer to the event object. Thanks to l4rk for figuring this out!
    jQuery.event.props.push('dataTransfer');

    // Do things depending on storage ID existance.
    doThingsOnIDExistingCheck();

    // Set token first if token hash (#access_token=...) is existing.
    if (window.location.hash) {
        setToken();
    }

    // ...otherwise do everything else!
    else {
        // Get action and storage ID from URL param.
        var action = $.query.get('action');
        dsid = $.query.get('id');

        // Check for additional param in action string, PDP seems to prevail only the first param in the list!
        if (action.indexOf('-') > 0) {
            var split = action.split('-');
            prefix = split[0];
            suffix = split[1];
            // Handle 'create-123456' things (old storage IDs).
            if (prefix === 'create' && isOldStorageID(suffix)) {
                dsid = suffix;
                migrateOldDataset(dsid);
            }
            // Handle loading files from file dialog or drag'n'drop.
            else if (prefix === 'create' && suffix === 'files') {
                var fileData = sessionStorage.getItem('files');
                csvToTable(fileData);
                postToDariahStorage(fileData);
                // Wrong? Was: sessionStorage.remove('files');
                sessionStorage.removeItem('files');
            }
        }
        // New datasheet shall be created.
        else if (action === 'create') {
            createNew();
        }
        // Datasheet shall be uhared.
        else if (action === 'share') {
            shareDataset();
        }
        // Datasheet shall be unshared.
        else if (action === 'unshare') {
            unshareDataset();
        }
        // Datasheet shall be deleted.
        else if (action === 'delete') {
            deleteDataset();
        }
        // Autosave after authentication.
        else if (action === 'autosave') {
            autoSave();
        }
        // If we have got no action but an ID in the URL params.
        else {
            if (dsid !== '') {
                // Check if we have got an old storage ID here, load data from old DARIAH-DE Storage then.
                if (isOldStorageID(dsid)) {
                    loadFromOldDariahStorage(dsid);
                } else {
                    // Load data from OwnStorage if ID supplied in params.
		            loadFromDariahStorage(dsid);
                    if (sessionStorage.getItem('migration') === 'true') {
                        sessionStorage.removeItem('migration');
                        var url = host + '/edit/index.html?id=' + dsid;
                        var title = 'Dataset was successfully migrated to the DARIAH-DE OwnStorage!';
                        var message = '<p>The migrated dataset has got the ID <i>' + dsid + '</i> and is now stored in the DARIAH-DE OwnStorage. You can access it here in the Datasheet Editor using the URL: ' + '<a href="'+ url + '">' + url + '</a> and you can now do everything you can do with your DARIAH-DE OwnStorage datasets :-) Have fun!</p>';
                        newAlert('success', title,  message);
                    }
                }
            }
            // If we have got no ID in the URL params (no create action).
        	else {
	            showFirstStartWizard();
    	    }
    	}
	}
}

/**
 * Ask for creation (modal) if a dsid is already existing.
 */
function createNew() {
    if (sessionStorage.getItem('tok') !== null) {
        if (dsid && !isOldStorageID(dsid)) {
            $('#askForCreation').modal('show');
        } else {
            createNewDataset();
        }
    } else {
        var action = 'create';
        if (isOldStorageID(dsid)) {
            action += '-' + dsid;
        }
        authenticate('action=' + action);
    }
}

/**
 * Create new dataset.
 */
function createNewDataset() {
    // Create new dataset from old storage.
    if (isOldStorageID(dsid)) {
        // Gat data out of table.
        var csv = arr2csv($('#dataTable').handsontable('getDataReference'));
        // Create new data file.
        postToDariahStorage(csv);
    }
    // Create new dataset with empty table body.
    else {
        // Re-set ID.
        dsid = '';
        // Create the new handsontable here.
        $('#dataTable').handsontable('loadData', [e4dHeader]);
        // Put table to csv array and create CSV structure.
        var arr = arr2csv($('#dataTable').handsontable('getDataReference'));
        // Create new data file.
        postToDariahStorage(arr);
    }
}

/*
 *
 */
function deleteDataset() {
    if (sessionStorage.getItem('tok') !== null) {
        $('#askForDeletion').modal('show');
    } else {
        authenticate('action=delete');
    }
}

/*
 *
 */
function shareDataset() {
    if (sessionStorage.getItem('tok') !== null) {
        $('#askForSharing').modal('show');
    } else {
        authenticate('action=share');
    }
}

/*
 *
 */
function unshareDataset() {
    if (sessionStorage.getItem('tok') !== null) {
        $('#askForUnsharing').modal('show');
    } else {
        authenticate('action=unshare');
    }
}

/*
 * If we can read the dataset without token, it is public :-)
 */
function checkPublicAccess() {
    if (sessionStorage.getItem('tok') !== null) {
        var response = checkStorageAccess('delete');
    } else {
        authenticate('action=unshare');
    }
}

/**
 *
 */
var saveTimer;
function autoSave(change) {
    if (autosave) {
        // Get data.
        var arr = $('#dataTable').handsontable('getDataReference');
    	var csv = arr2csv(arr);
        // Make sure that not more than one save per 2,5 seconds occur.
        clearTimeout(saveTimer);
        saveTimer = setTimeout(function() {
            // Check for token first.
            if (sessionStorage.getItem('tok') !== null) {
                updateDariahStorage(dsid, csv);
            } else {
                authenticate('action=autosave');
            }
        }, 2500);
    }
}

/**
 * Functions for loading file into Browser, drag'n'drop or with button
 * look at
 * - http://www.html5rocks.com/de/tutorials/file/dndfiles/
 * - http://weblog.bocoup.com/using-datatransfer-with-jquery-events/
 */

/**
 *
 */
function handleFileSelect(evt) {
	var files = evt.target.files; // FileList object
	loadFromFiles(files);
}

/**
 *
 */
function handleDropFileSelect(evt) {
	evt.stopPropagation();
	evt.preventDefault();
	var files = evt.dataTransfer.files; // FileList object.
	loadFromFiles(files);
}

/**
 *
 */
function handleDragOver(evt) {
	evt.stopPropagation();
	evt.preventDefault();
	evt.dataTransfer.dropEffect = 'copy'; // Explicitly show this is a copy.
}

/**
 *
 */
function loadFromFiles(files) {
    for (var i = 0, f; f = files[i]; i++) {
        //console.log(f.name + ' - ' + f.type || 'n/a' + ' - ' + f.size + ' bytes, last modified: ' + f.lastModifiedDate.toLocaleDateString());
        if($.inArray(f.type, allowedMimeTypes) < 0 ) {
            newAlert('error', 'Could not open or handle file!', 'Maybe the Mime-Type "' + f.type + '" is not supported or your file is not a CSV file. Datasheet Editor can only handle CSV files. If your file really is a CSV file, maybe it is missing a .csv file suffix?', '');
            continue;
        }
        var reader = new FileReader();
        reader.onload = function(e) {
            var data = e.target.result;
            if (sessionStorage.getItem('tok') !== null) {
                csvToTable(data);
                postToDariahStorage(data);
            } else {
                // Cache data.
                sessionStorage.setItem('files', data);
                authenticate('action=create-files');
            }
        };
        reader.readAsText(f);
    }
}

/**
 *
 */
function csvToTable(data) {
	var arr = jQuery.csv()(data);
	// First turn off autosave.
	autosave = false;
	$('#dataTable').handsontable('loadData', arr);
	// colHeaders kind of broken...? this fixes, but what's the reason?
	$('#dataTable').handsontable('updateSettings', {colHeaders: true, rowHeaders: true});
    // Turn on autosave again.
    autosave = true;
	hideFirstStartWizard();
}

/**
 *
 */
function csvToReadOnlyTable(data) {
    initReadOnlyTable();
	var arr = jQuery.csv()(data);
	// First turn off autosave.
	autosave = false;
	$('#dataTable').handsontable('loadData', arr);
	// colHeaders kind of broken...? this fixes, but what's the reason?
	$('#dataTable').handsontable('updateSettings', {colHeaders: true, rowHeaders: true});
    // Do NOT turn autosave on, we cannot save to old storage!
	hideFirstStartWizard();
    showMessage('console', 'This dataset is read-only.');
    showMessage('idMessage', 'This dataset was loaded from the old DARIAH-DE Storage and has got the ID');
}

/**
 * Just downloads dataset from table.
 */
function download() {
    // Get URL for filename and download.
    var url;
    if (isOldStorageID(dsid)) {
        url = oldStorageURL + dsid;
    } else {
        url = storageURL + dsid;
    }
    // Create new a element.
    var fancyA = document.createElement('a');
    document.body.appendChild(fancyA);
    fancyA.style = 'display:none';
    // Gat data out of table, set URL.
    var csv = arr2csv($('#dataTable').handsontable('getDataReference'));
    var blob = new Blob([csv], {type: 'text/csv;charset=utf-8'});
    url = window.URL.createObjectURL(blob);
    // Fill a element with URL and filename.
    fancyA.href = url;
    fancyA.download = dsid + '.csv';
    // Click on a :-)
    fancyA.click();
    // Remove URL.
    window.URL.revokeObjectURL(url);
}

/**
 *
 */
function showFirstStartWizard() {
	$('#firstStartWizard').show();
	$('#sheetArea').hide();
    showMessage('console', '');
    showMessage('idMessage', '');
    showMessage('sharedStatus', '');
    // Empty tgn list.
    $('#tgnList').empty();
    // TODO: Empty map data, too?
}

/**
 *
 */
function hideFirstStartWizard() {
	$('#firstStartWizard').hide();
	$('#sheetArea').show();
    showMessage('console', 'All changes will be autosaved.');
    showMessage('idMessage', 'Your dataset is stored to the DARIAH-DE OwnStorage and has got the ID');
	// Check if data in sheet is fine.
	//checkColumnHeaders();
}

/**
 *
 */
function newSheetAlert() {
	hideFirstStartWizard();
    var url = host + '/edit/index.html?id=' + dsid;
	var title = 'New datasheet created!';
	var message = '<p>Your datasheet has been successfully created and stored to the DARIAH-DE OwnStorage. '
        + 'You can access it here in the Datasheet Editor using the URL: ' + '<a href="'+ url + '">' + url + '</a></p>';
	newAlert('success', title,  message);
}

/**
 *
 */
function openGeoBrowser(newWindow) {
    // Check all at once.
    var val1 = checkColumnHeaders();
    var val2 = checkDates();
    var val3 = checkCoordinates();
    if (! (val1 && val2 && val3) ) {
        return;
    }

    var url = host + '?csv1=';
    if (isOldStorageID(dsid)) {
        // TODO Do we really need to URIencode here?
        //url += encodeURIComponent(oldStorageURL + dsid);
        url += oldStorageURL + dsid;
    } else {
        // TODO Do we need to URIencode here?
        //url += encodeURIComponent(storageURL + dsid);
        url += storageURL + dsid;
    }

	if (newWindow) {
		window.open(url);
	} else {
		document.location.href = url;
	}
}

 /**
 *
 */
function showMessage(area, text) {
    $('#' + area).text(text);
}

/**
 *
 */
function doThingsOnIDExistingCheck () {
    if (dsid) {
        checkSharedStatus(dsid);
        url = storageURL + dsid;
        $('#dataLoc input').val(dsid);
        $('#dataLoc button').data('clipboard-text', url);
        $('#dataLoc button').attr('data-clipboard-text', url);
        $('#dataLoc button').removeClass('disabled');
        $('#dataLoc button').attr('disabled', false);
        $('#uploadButton').css('display','none');
        enableManagedButtons();
    } else {
        $('#dataLoc input').val('ID not (yet) set');
        $('#dataLoc button').addClass('disabled');
        disableManagedButtons();
    }
}

/**
 *
 */
function doThingsOnOldStorageID () {
    if (dsid) {
        // Old storage ID is always available, so we can put the URL in here.
        var url = oldStorageURL + dsid;
        $('#dataLoc input').val(dsid);
        $('#dataLoc button').data('clipboard-text', url);
        $('#dataLoc button').attr('data-clipboard-text', url);
        $('#dataLoc button').removeClass('disabled');
        $('#dataLoc button').attr('disabled', false);
        // Enable download button only.
        $('#downloadButton').removeClass('disabled');
        $('#downloadButton').attr('disabled', false);
        // Disable other buttons.
        $('#deleteButton').addClass('disabled');
        $('#deleteButton').attr('disabled', true);
        $('#shareButton').addClass('disabled');
        $('#shareButton').attr('disabled', true);
        $('#unshareButton').addClass('disabled');
        $('#unshareButton').attr('disabled', true);
        // Disable Geolocation completion button.
        $('#geolocationCompletionButton').addClass('disabled');
        $('#geolocationCompletionButton').attr('disabled', true);
        // Disable Set button in map selection.
        $('#setAllPlacesOSMButton').addClass('disabled');
        $('#setAllPlacesOSMButton').attr('disabled', true);
        // Enable Geo-Browser button.
        $('#openWithGeoBrowserButton').removeClass('disabled');
        $('#openWithGeoBrowserButton').attr('disabled', false);
    }
}

/**
 *
 */
function disableManagedButtons() {
    // Disable buttons and make them un-clickable.
    $('#downloadButton').addClass('disabled');
    $('#downloadButton').attr('disabled', true);
    $('#deleteButton').addClass('disabled');
    $('#deleteButton').attr('disabled', true);
    $('#shareButton').addClass('disabled');
    $('#shareButton').attr('disabled', true);
    $('#unshareButton').addClass('disabled');
    $('#unshareButton').attr('disabled', true);
    $('#geolocationCompletionButton').addClass('disabled');
    $('#geolocationCompletionButton').attr('disabled', true);
    $('#openWithGeoBrowserButton').addClass('disabled');
    $('#openWithGeoBrowserButton').attr('disabled', true);
    $('#setAllPlacesOSMButton').addClass('disabled');
    $('#setAllPlacesOSMButton').attr('disabled', true);
    // Change type of buttons (color) as guide for the users.
    $('#newButton').addClass('btn-primary');
    $('#importButton').addClass('btn-primary');
    $('#downloadButton').removeClass('btn-primary');
    $('#deleteButton').removeClass('btn-danger');
    $('#shareButton').removeClass('btn-primary');
    $('#unshareButton').removeClass('btn-primary');
    $('#geolocationCompletionButton').removeClass('btn-primary');
    $('#openWithGeoBrowserButton').removeClass('btn-primary');
}

/**
 *
 */
function enableManagedButtons() {
    // Enable buttons and make them clickable.
    $('#downloadButton').removeClass('disabled');
    $('#downloadButton').attr('disabled', false);
    $('#deleteButton').removeClass('disabled');
    $('#deleteButton').attr('disabled', false);
    $('#shareButton').removeClass('disabled');
    $('#shareButton').attr('disabled', false);
    $('#unshareButton').removeClass('disabled');
    $('#unshareButton').attr('disabled', false);
    $('#geolocationCompletionButton').removeClass('disabled');
    $('#geolocationCompletionButton').attr('disabled', false);
    $('#openWithGeoBrowserButton').removeClass('disabled');
    $('#openWithGeoBrowserButton').attr('disabled', false);
    $('#setAllPlacesOSMButton').removeClass('disabled');
    $('#setAllPlacesOSMButton').attr('disabled', false);
    // Change type of buttons (color) as guide for the users.
    $('#newButton').removeClass('btn-primary');
    $('#importButton').removeClass('btn-primary');
    $('#downloadButton').addClass('btn-primary');
    $('#deleteButton').addClass('btn-danger');
    $('#shareButton').addClass('btn-primary');
    $('#unshareButton').addClass('btn-primary');
    $('#geolocationCompletionButton').addClass('btn-primary');
    $('#openWithGeoBrowserButton').addClass('btn-primary');
}

/**
 * Set shared status in session storage and on screen!
 */
function setSharedStatus() {
    sessionStorage.setItem('public', true);
    $('#shareButton').hide();
    $('#unshareButton').show();
    $('#sharedStatus').text('Your dataset is publicly shared');
}

/**
 * Set unshared status in session storage and on screen!
 */
function setUnsharedStatus() {
    sessionStorage.removeItem('public');
    $('#unshareButton').hide();
    $('#shareButton').show();
    $('#sharedStatus').text('Your dataset is private');
}

/*
 *
 */
function toggleUploadButton() {
    // Hiding button is always possible without warning.
    if (!document.getElementById('uploadButton').style.display) {
        $('#uploadButton').css('display','none');
    }
    // Showing the button reveals a warning, if a dataset is already on display.
    else {
        if (dsid && !isOldStorageID(dsid)) {
            $('#askForImport').modal('show');
        } else {
            $('#uploadButton').css('display','');
        }
    }
}

/*
 *
 */
function showUploadButton() {
    $('#uploadButton').css('display','');
}

/**
 *
 */
function initClipboard() {
    $('button').tooltip('hide');
    var clipboard = new ClipboardJS('.doi-copy');
    clipboard.on('success', function(e) {
        var originalTitle = e.trigger.getAttribute('data-original-title');
        $(e.trigger).tooltip('hide')
            .attr('data-original-title', 'Copied')
            .tooltip('show')
        e.clearSelection();
        setTimeout(function() {
            $(e.trigger).tooltip('hide').attr('data-original-title', originalTitle);
        }, 2000);
    });
}

/**
 * Returns true if given id is one of the old DARIAH Storage.
 */
function isOldStorageID(id) {
    return /^[0-9]+$/.test(id);
}

/*
 *
 */
function showNews() {
    var title = 'News';
    var message = '<p>Since the latest Geo-Browser release, the DARIAH-DE OwnStorage is used to store your datasets. The usage of the Geo-Browser and Datasheet editor now is more secure and you can work on a dataset privately until your dataset is complete and working, and then you can share it with the world.</p><p>To store your datasets to the DARIAH-DE OwnStorage, you will have to log in via the DARIAH AAI using your DARIAH or eduGAIN federation account.</p><p>More information about the new DARIAH-DE Datasheet Editor you can look up in the <strong><a href="https://geobrowser.de.dariah.eu/beta/doc/datasheet.html">WHAT IS NEW?</a></strong> documentation.</p>';
    newAlert('info', title, message, '', '', '', 'nodate');
}

/*
 *
 */
function toggleHowto() {
    var title = 'How to fill the table?';
    var message = "<dl><dt><strong style='color:green;'>Name</strong></dt> <dd>enables you to name your data point. Place names (used for <i>Geolocation completion</i>) have to be entered in the <i>Address</i> field, so <i>Name</i> can be used to display the name of an event or a name that would not be recognized as Address.</dd>" +
    "<dt><strong style='color:green;'>Address</strong></dt> <dd>identifies a place with this place name and will be used to add <i>Longitude</i> and <i>Latitude</i> via <i>Geolocation completion</i>.</dd>" +
    "<dt><strong style='color:green;'>Description</strong></dt> <dd>is used to display any additional Data.</li>" +
    "<dt><strong style='color:green;'>Longitude</strong> and <strong style='color:green;'>Latitude</strong></dt> <dd>can be filled in by <i>Geolocation completion</i> or manually entered and uses decimal geographic coordinates. Existing coordinates will not be overwritten by <i>Geolocation completion</i>.</dd>" +
    "<dt><strong style='color:green;'>TimeStamp</strong></dt> <dd>is used to store the date of an entry. The format for time stamps is YYYY-MM-DD, however just naming the year is also possible.</dd>" +
    "<dt><strong style='color:green;'>TimeSpan:begin</strong> and <strong style='color:green;'>TimeSpan:end</strong></dt> <dd>can be used to convey a timespan. Just like <i>TimeStamp</i> it uses YYYY-MM-DD but can be used without specifying the day.</dd>" +
    "<dt><strong style='color:green;'>GettyID</strong></dt> <dd>is automatically filled in using the Getty Thesaurus of Geographic Names via <i>Geolocation completion</i>. An existing GettyID will not be overwritten by <i>Geolocation completion</i>.</dd></dl>";

    if (document.getElementById('howto')) {
        document.getElementById('howto').remove();
    } else {
        newAlert('info', title, message, '', 'howto', 'howtoArea', 'nodate');
    }
}

/**
 *
 */
function logWidth() {
    console.log("offwi: " + $('#sheetArea')[0].offsetWidth);
    console.log("wi: " + $('#sheetArea').width());
    //console.log("wi css: " + $('#console').css(width));
    console.log("owwi: " + $('#sheetArea').outerWidth(true));
    console.log("dt offwi: " + $('#dataTable')[0].offsetWidth);
    console.log("dt wi: " + $('#dataTable').width());
    //console.log("wi css: " + $('#console').css(width));
    console.log("dt owi: " + $('#dataTable').outerWidth(true));
}

/**
 * Gets file list data from PDP and displays them in a new window/tab (ID, shared/unshared).
 * User should be able to show, delete, share/unshare every object from the list.
 */
function showFileList(data) {
    // **TODO**
    // Maybe add and sort on creationDate..?
}
